@@ -89,6 +89,7 @@ struct Parse_Options_s
 	int display_postscript;       /* if true, output postscript linkage */
 	int display_constituents;     /* if true, output treebank-style constituent structure */
 	int display_bad;              /* if true, bad linkages are displayed */
+	int display_disjuncts;        /* if true, print disjuncts that were used */
 	int display_links;            /* if true, a list o' links is printed out */
 	int display_senses;           /* if true, sense candidates are printed out */
 };
@@ -103,7 +104,7 @@ typedef struct
 struct Dictionary_s
 {
 	Dict_node *     root;
-	char *          name;
+	const char *    name;
 
 	int             use_unknown_word;
 	int             unknown_word_defined;
@@ -122,9 +123,22 @@ struct Dictionary_s
 
 	int             left_wall_defined;
 	int             right_wall_defined;
+
+	/* Affixes are used during the tokenization stage. */
+	Dictionary      affix_table;
+	int r_strippable; /* right */
+	int l_strippable; /* left */
+	int u_strippable; /* units on left */
+	int s_strippable; /* generic suffix */
+	int p_strippable; /* generic prefix */
+	const char ** strip_left;
+	const char ** strip_right;
+	const char ** strip_units;
+	const char ** prefix;
+	const char ** suffix;
+
 	Postprocessor * postprocessor;
 	Postprocessor * constituent_pp;
-	Dictionary      affix_table;
 	int             andable_defined;
 	Connector_set * andable_connector_set;  /* NULL=everything is andable */
 	Connector_set * unlimited_connector_set; /* NULL=everthing is unlimited */
@@ -143,10 +157,10 @@ struct Dictionary_s
 	 */
 	FILE *          fp;
 	char            token[MAX_TOKEN_LENGTH];
-	int             is_special;
-	int             already_got_it;
+	int             is_special;        /* boolean */
+	wint_t          already_got_it;
 	int             line_number;
-	int             recursive_error;
+	int             recursive_error;   /* boolean */
 };
 
 struct Label_node_s
@@ -263,7 +277,7 @@ struct PP_data_s
 struct PP_info_s
 {
 	int       num_domains;
-	char **   domain_name;
+	const char **   domain_name;
 };
 
 struct Postprocessor_s
