@@ -97,6 +97,7 @@ Parse_Options parse_options_create(void)
 	po->display_postscript = FALSE;
 	po->display_constituents = 0;
 	po->display_bad = FALSE;
+	po->display_disjuncts = FALSE;
 	po->display_links = FALSE;
 	po->display_senses = FALSE;
 
@@ -314,6 +315,14 @@ int parse_options_get_display_bad(Parse_Options opts) {
 	return opts->display_bad;
 }
 
+void parse_options_set_display_disjuncts(Parse_Options opts, int dummy) {
+	opts->display_disjuncts = dummy;
+}
+
+int parse_options_get_display_disjuncts(Parse_Options opts) {
+	return opts->display_disjuncts;
+}
+
 void parse_options_set_display_links(Parse_Options opts, int dummy) {
 	opts->display_links = dummy;
 }
@@ -369,6 +378,123 @@ void parse_options_reset_resources(Parse_Options opts) {
 *
 ****************************************************************/
 
+/* Units will typically have a ".u" at the end. Get
+ * rid of it, as otherwise stipping is messed up. */
+static inline char * deinflect(const char * str)
+{
+	char * s = strdup(str);
+	char * p = strchr(s, '.');
+	if (p && p != s) *p = 0x0;
+	return s;
+}
+
+static void affix_list_create(Dictionary dict)
+{
+	int i, j, k, l, m;
+	int r_strippable=0, l_strippable=0, u_strippable=0;
+	int s_strippable=0, p_strippable=0;
+	Dict_node * dn, * dn2, * start_dn;
+
+	const char * rpunc_con = "RPUNC";
+	const char * lpunc_con = "LPUNC";
+	const char * units_con = "UNITS";
+
+	/* Hmm SUF and PRE do not seem to be used at this time ... */
+	const char * suf_con = "SUF";
+	const char * pre_con = "PRE";
+
+	dict->strip_left = NULL;
+	dict->strip_right = NULL;
+	dict->strip_units = NULL;
+	dict->prefix = NULL;
+	dict->suffix = NULL;
+
+	/* Load affixes from the affix table. 
+	 */
+	start_dn = list_whole_dictionary(dict->root, NULL);
+	for (dn = start_dn; dn != NULL; dn = dn->right)
+	{
+		if (word_has_connector(dn, rpunc_con, 0)) r_strippable++;
+		if (word_has_connector(dn, lpunc_con, 0)) l_strippable++;
+		if (word_has_connector(dn, units_con, 0)) u_strippable++;
+		if (word_has_connector(dn, suf_con, 0)) s_strippable++;
+		if (word_has_connector(dn, pre_con, 0)) p_strippable++;
+	}
+	dict->strip_right = (const char **) xalloc(r_strippable * sizeof(char *));
+	dict->strip_left = (const char **) xalloc(l_strippable * sizeof(char *));
+	dict->strip_units = (const char **) xalloc(u_strippable * sizeof(char *));
+	dict->suffix = (const char **) xalloc(s_strippable * sizeof(char *));
+	dict->prefix = (const char **) xalloc(p_strippable * sizeof(char *));
+
+	dict->r_strippable = r_strippable;
+	dict->l_strippable = l_strippable;
+	dict->u_strippable = u_strippable;
+	dict->p_strippable = p_strippable;
+	dict->s_strippable = s_strippable;
+
+	i = 0;
+	j = 0;
+	k = 0;
+	l = 0;
+	m = 0;
+	dn = start_dn;
+
+	while (dn != NULL)
+	{
+		if (word_has_connector(dn, rpunc_con, 0))
+		{
+			dict->strip_right[i] = deinflect(dn->string);
+			i++;
+		}
+		if (word_has_connector(dn, lpunc_con, 0))
+		{
+			dict->strip_left[j] = deinflect(dn->string);
+			j++;
+		}
+		if (word_has_connector(dn, units_con, 0))
+		{
+			dict->strip_units[m] = deinflect(dn->string);
+			m++;
+		}
+		if (word_has_connector(dn, suf_con, 0))
+		{
+			dict->suffix[k] = dn->string;
+			k++;
+		}
+		if (word_has_connector(dn, pre_con, 0))
+		{
+			dict->prefix[l] = dn->string;
+			l++;
+		}
+		dn2 = dn->right;
+		dn->right = NULL;
+		xfree(dn, sizeof(Dict_node));
+		dn = dn2;
+	}
+}
+
+static void affix_list_delete(Dictionary dict)
+{
+	int i;
+	for (i=0; i<dict->l_strippable; i++)
+	{
+		free((char *)dict->strip_left[i]);
+	}
+	for (i=0; i<dict->r_strippable; i++)
+	{
+		free((char *)dict->strip_right[i]);
+	}
+	for (i=0; i<dict->u_strippable; i++)
+	{
+		free((char *)dict->strip_units[i]);
+	}
+	xfree(dict->strip_right, dict->r_strippable * sizeof(char *));
+	xfree(dict->strip_left, dict->l_strippable * sizeof(char *));
+	xfree(dict->strip_units, dict->u_strippable * sizeof(char *));
+	xfree(dict->suffix, dict->s_strippable * sizeof(char *));
+	xfree(dict->prefix, dict->p_strippable * sizeof(char *));
+}
+
 /**
  * The following function is dictionary_create with an extra
  * paramater called "path". If this is non-null, then the path
@@ -378,8 +504,8 @@ void parse_options_reset_resources(Parse_Options opts) {
  * function.
  */
 Dictionary
-dictionary_create(char * dict_name, char * pp_name,
-                  char * cons_name, char * affix_name)
+dictionary_create(const char * dict_name, const char * pp_name,
+                  const char * cons_name, const char * affix_name)
 {
 	Dictionary dict;
 	Dict_node *dict_node;
@@ -415,11 +541,14 @@ dictionary_create(char * dict_name, char * pp_name,
 	fclose(dict->fp);
 
 	dict->affix_table = NULL;
-	if (affix_name != NULL) {
+	if (affix_name != NULL)
+	{
 		dict->affix_table = dictionary_create(affix_name, NULL, NULL, NULL);
-		if (dict->affix_table == NULL) {
+		if (dict->affix_table == NULL)
+		{
 			goto failure;
 		}
+		affix_list_create(dict->affix_table);
 	}
 
 	dict->left_wall_defined  = boolean_dictionary_lookup(dict, LEFT_WALL_WORD);
@@ -513,6 +642,7 @@ int dictionary_delete(Dictionary dict)
 	}
 
 	if (dict->affix_table != NULL) {
+		affix_list_delete(dict->affix_table);
 		dictionary_delete(dict->affix_table);
 	}
 
@@ -541,21 +671,9 @@ int dictionary_get_max_cost(Dictionary dict)
 
 static Linkage_info * linkage_info_new(int num_to_alloc)
 {
-#ifdef USE_CORPUS
-	int i,j;
-#endif
 	Linkage_info *link_info;
 	link_info = (Linkage_info *) xalloc(num_to_alloc * sizeof(Linkage_info));
-#ifdef USE_CORPUS
-	for (i=0; i<num_to_alloc; i++)
-	{
-		Linkage_info *lifo = &link_info[i];
-		for (j=0; j<MAX_SENTENCE; j++)
-		{
-			lifo->disjunct_list_str[j] = NULL;
-		}
-	}
-#endif
+	memset(link_info, 0, num_to_alloc * sizeof(Linkage_info));
 	return link_info;
 }
 
@@ -815,7 +933,7 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 *
 ****************************************************************/
 
-Sentence sentence_create(char *input_string, Dictionary dict)
+Sentence sentence_create(const char *input_string, Dictionary dict)
 {
 	Sentence sent;
 	int i;
@@ -895,7 +1013,13 @@ int sentence_length(Sentence sent)
 	return sent->length;
 }
 
-char * sentence_get_word(Sentence sent, int index)
+const char * sentence_get_word(Sentence sent, int index)
+{
+	if (!sent) return NULL;
+	return sent->word[index].string;
+}
+
+const char * sentence_get_nth_word(Sentence sent, int index)
 {
 	if (!sent) return NULL;
 	return sent->word[index].string;
@@ -941,12 +1065,8 @@ int sentence_link_cost(Sentence sent, int i) {
 	return sent->link_info[i].link_cost;
 }
 
-char * sentence_get_nth_word(Sentence sent, int i) {
-	if (!sent) return NULL;
-	return sent->word[i].string;
-}
-
-int sentence_nth_word_has_disjunction(Sentence sent, int i) {
+int sentence_nth_word_has_disjunction(Sentence sent, int i)
+{
 	if (!sent) return 0;
 	return (sent->parse_info->chosen_disjuncts[i] != NULL);
 }
@@ -1071,13 +1191,8 @@ int linkage_set_current_sublinkage(Linkage linkage, int index)
 
 static void exfree_pp_info(PP_info *ppi)
 {
-	int i;
-	for (i=0; i<ppi->num_domains; ++i)
-	{
-		exfree(ppi->domain_name[i], strlen(ppi->domain_name[i])+1);
-	}
 	if (ppi->num_domains > 0)
-		exfree(ppi->domain_name, sizeof(char *)*ppi->num_domains);
+		exfree(ppi->domain_name, sizeof(const char *)*ppi->num_domains);
 	ppi->domain_name = NULL;
 	ppi->num_domains = 0;
 }
@@ -1138,17 +1253,18 @@ static int link_already_appears(Linkage linkage, Link *link, int a)
 	return FALSE;
 }
 
-static PP_info excopy_pp_info(PP_info ppi) {
-	 static PP_info newppi;
-	 int i;
+static PP_info excopy_pp_info(PP_info ppi)
+{
+	static PP_info newppi;
+	int i;
 
-	 newppi.num_domains = ppi.num_domains;
-	 newppi.domain_name = (char **) exalloc(sizeof(char *)*ppi.num_domains);
-	 for (i=0; i<newppi.num_domains; ++i) {
-		 newppi.domain_name[i] = (char *) exalloc(sizeof(char)*(strlen(ppi.domain_name[i])+1));
-		 strcpy(newppi.domain_name[i], ppi.domain_name[i]);
-	 }
-	 return newppi;
+	newppi.num_domains = ppi.num_domains;
+	newppi.domain_name = (const char **) exalloc(sizeof(const char *)*ppi.num_domains);
+	for (i=0; i<newppi.num_domains; ++i)
+	{
+		newppi.domain_name[i] = ppi.domain_name[i];
+	}
+	return newppi;
 }
 
 
@@ -1337,7 +1453,7 @@ Sentence linkage_get_sentence(Linkage linkage)
 
 const char * linkage_get_disjunct(Linkage linkage, int w)
 {
-	if (NULL == linkage->info->disjunct_list_str)
+	if (NULL == linkage->info->disjunct_list_str[w])
 	{
 		lg_compute_disjunct_strings(linkage->sent, linkage->info);
 	}
@@ -1472,13 +1588,15 @@ void linkage_post_process(Linkage linkage, Postprocessor * postprocessor)
 				subl->pp_info[j].num_domains = k;
 				if (k > 0)
 				{
-					subl->pp_info[j].domain_name = (char **) exalloc(sizeof(char *)*k);
+					subl->pp_info[j].domain_name = (const char **) exalloc(sizeof(const char *)*k);
 				}
 				k = 0;
 				for (d = pp->d_type_array[j]; d != NULL; d = d->next)
 				{
-					subl->pp_info[j].domain_name[k] = (char *) exalloc(sizeof(char)*2);
-					sprintf(subl->pp_info[j].domain_name[k], "%c", d->type);
+					char buff[5];
+					sprintf(buff, "%c", d->type);
+					subl->pp_info[j].domain_name[k] = string_set_add (buff, sent->string_set);
+
 					k++;
 				}
 			}
