@@ -62,7 +62,7 @@ Corpus * lg_corpus_new(void)
 			sqlite3_errmsg(c->dbconn));
 	}
 
-	prt_error("Info: Statistics database opened\n");
+	prt_error("Info: linkgrammar corpus statistics database opened\n");
 	return c;
 }
 
@@ -241,7 +241,7 @@ void lg_corpus_score(Corpus *corp, Sentence sent, Linkage_info *lifo)
 	/* No-op if the database is not open */
 	if (NULL == corp->dbconn) return;
 
-	if (NULL == lifo->disjunct_list_str)
+	if (NULL == lifo->disjunct_list_str[1])
 	{
 		lg_compute_disjunct_strings(sent, lifo);
 	}
