@@ -11,9 +11,9 @@
 /*                                                                       */
 /*************************************************************************/
 
-s64  table_lookup(int, int, Connector *, Connector *, int);
-int  match(Connector *a, Connector *b, int wa, int wb);
-int  x_match(Connector *a, Connector *b);
+s64  table_lookup(Sentence, int, int, Connector *, Connector *, int);
+int  match(Sentence, Connector *a, Connector *b, int wa, int wb);
+int  x_match(Sentence, Connector *a, Connector *b);
 void init_table(Sentence sent);
 void free_table(Sentence sent);
 s64  parse(Sentence sent, int mincost, Parse_Options opts);
@@ -22,3 +22,6 @@ void count_set_effective_distance(Sentence sent);
 void count_unset_effective_distance(Sentence sent);
 void delete_unmarked_disjuncts(Sentence sent);
 
+void init_count(Sentence sent);
+void free_count(Sentence sent);
+
