@@ -33,6 +33,7 @@
  *
  ****************************************************************************/
 
+#include <errno.h>
 #include <locale.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -77,6 +78,14 @@ fget_input_string(FILE *in, FILE *out, Parse_Options opts)
 	static char * pline = NULL;
 	const char * prompt = "linkparser> ";
 
+	if (in != stdin)
+	{
+		static char input_string[MAX_INPUT];
+		input_pending = FALSE;
+		if (fgets(input_string, MAX_INPUT, in)) return input_string;
+		return NULL;
+	}
+
 	if (input_pending && pline != NULL)
 	{
 		input_pending = FALSE;
@@ -197,6 +206,12 @@ static void process_linkage(Linkage linkage, Parse_Options opts)
 			fprintf(stdout, "%s", string);
 			linkage_free_senses(string);
 		}
+		if (parse_options_get_display_disjuncts(opts))
+		{
+			string = linkage_print_disjuncts(linkage);
+			fprintf(stdout, "%s", string);
+			linkage_free_disjuncts(string);
+		}
 		if (parse_options_get_display_postscript(opts))
 		{
 			string = linkage_print_postscript(linkage, FALSE);
@@ -381,12 +396,13 @@ static int special_command(char *input_string, Dictionary dict)
 	if (input_string[0] == '\n') return TRUE;
 	if (input_string[0] == COMMENT_CHAR) return TRUE;
 	if (input_string[0] == '!') {
-		if (strncmp(input_string, "!panic_", 7)==0) {
+		if (strncmp(input_string, "!panic_", 7) == 0)
+		{
 			issue_special_command(input_string+7, panic_parse_opts, dict);
+			return TRUE;
 		}
-		else {
-			issue_special_command(input_string+1, opts, dict);
-		}
+
+		issue_special_command(input_string+1, opts, dict);
 		return TRUE;
 	}
 	return FALSE;
@@ -423,12 +439,15 @@ static void setup_panic_parse_options(Parse_Options opts)
 static void print_usage(char *str) {
 	fprintf(stderr,
 			"Usage: %s [language]\n"
-			"		  [-ppoff] [-coff] [-aoff] [-batch] [-<special \"!\" command>]\n", str);
+			"		  [-ppoff] [-coff] [-aoff] [-batch]\n"
+			"		  [-<special \"!\" command>]\n"
+			"		  [--version]\n", str);
 	exit(-1);
 }
 
 int main(int argc, char * argv[])
 {
+	FILE            *input_fh = stdin;
 	Dictionary      dict;
 	Sentence        sent;
 	const char     *language="en";  /* default to english, and not locale */
@@ -453,7 +472,10 @@ int main(int argc, char * argv[])
 
 	for (; i<argc; i++) {
 		if (argv[i][0] == '-') {
-			if (strcmp("-ppoff", argv[i])==0) {
+			if (strcmp("--version", argv[i])==0) {
+				printf("Version: %s\n", linkgrammar_get_version());
+				exit(0);
+			} else if (strcmp("-ppoff", argv[i])==0) {
 				pp_on = FALSE;
 			} else if (strcmp("-coff", argv[i])==0) {
 				cons_on = FALSE;
@@ -516,11 +538,38 @@ int main(int argc, char * argv[])
 	verbosity = parse_options_get_verbosity(opts);
 
 	/* Main input loop */
-	while (NULL != (input_string = fget_input_string(stdin, stdout, opts)))
+	while (1)
 	{
+		input_string = fget_input_string(input_fh, stdout, opts);
+
+		if (NULL == input_string)
+		{
+			if (input_fh == stdin) break;
+			fclose (input_fh);
+			input_fh = stdin;
+			continue;
+		}
+
 		if ((strcmp(input_string, "quit\n")==0) ||
 			(strcmp(input_string, "exit\n")==0)) break;
 
+		/* We have to handle the !file command inline; its too hairy
+		 * otherwise ... */
+		if (strncmp(input_string, "!file", 5) == 0)
+		{
+			char * filename = &input_string[6];
+			input_fh = fopen(filename, "r");
+			if (NULL == input_fh)
+			{
+				int perr = errno;
+				fprintf(stderr, "Error: %s (%d) %s\n",
+				        filename, perr, strerror(perr));
+				input_fh = stdin;
+				continue;
+			}
+			continue;
+		}
+
 		if (special_command(input_string, dict)) continue;
 		if (parse_options_get_echo_on(opts)) {
 			printf("%s", input_string);
