@@ -21,6 +21,8 @@
 
 #define MAX_STRIP 10
 
+/* These are no longer in use, but are read from the 4.0.affix file */
+/* I've left these here, as an axample of what to expect. */
 /*static char * strip_left[] = {"(", "$", "``", NULL}; */
 /*static char * strip_right[] = {")", "%", ",", ".", ":", ";", "?", "!", "''", "'", "'s", NULL};*/
 
@@ -48,12 +50,13 @@ static int is_initials_word(const char * word)
  */
 static int is_number(const char * s)
 {
+	int nb = 1;
 	wchar_t c;
 	if (!is_utf8_digit(s)) return FALSE;
 
-	while (*s != 0)
+	while ((*s != 0) && (0 < nb))
 	{
-		int nb = mbtowc(&c, s, 4);
+		nb = mbtowc(&c, s, MB_CUR_MAX);
 		if (iswdigit(c)) { s += nb; }
 
 		/* U+00A0 no break space */
@@ -80,7 +83,8 @@ static int ishyphenated(const char * s)
 
 	while (*s != '\0')
 	{
-		int nb = mbtowc(&c, s, 4);
+		int nb = mbtowc(&c, s, MB_CUR_MAX);
+		if (0 > nb) break;
 
 		if (!iswalpha(c) && !iswdigit(c) && (*s!='.') && (*s!=',')
 			&& (*s!='-')) return FALSE;
@@ -201,7 +205,7 @@ static int issue_sentence_word(Sentence sent, const char * s)
 		   If it's there, the process terminates.
 
 		   If it's not there and it ends in one of the right strippable
-		   strings (see "right_strip") then remove the strippable string
+		   strings (see "strip_right") then remove the strippable string
 		   and make it into a separate word.
 
 		   If there is no strippable string, then the process terminates.
@@ -235,7 +239,7 @@ static int downcase_is_in_dict(Dictionary dict, char * word)
 
 	if (!is_utf8_upper(word)) return FALSE;
 
-	nbh = mbtowc (&c, word, 4);
+	nbh = mbtowc (&c, word, MB_CUR_MAX);
 	c = towlower(c);
 	nbl = wctomb_check(low, c);
 	if (nbh != nbl)
@@ -258,83 +262,61 @@ static int downcase_is_in_dict(Dictionary dict, char * word)
  * w points to a string, wend points to the char one after the end.  The
  * "word" w contains no blanks.  This function splits up the word if
  * necessary, and calls "issue_sentence_word()" on each of the resulting
- * parts.  The process is described above.  returns TRUE of OK, FALSE if
+ * parts.  The process is described above.  Returns TRUE if OK, FALSE if
  * too many punctuation marks 
  */
 static int separate_word(Sentence sent, char *w, char *wend, int is_first_word, int quote_found)
 {
-	int i, j, k, l, len;
-	int r_strippable=0, l_strippable=0;
+	int i, j, len;
+	int r_strippable=0, l_strippable=0, u_strippable=0;
 	int s_strippable=0, p_strippable=0;
 	int  n_r_stripped, s_stripped;
 	int word_is_in_dict, s_ok;
-	int r_stripped[MAX_STRIP];  /* these were stripped from the right */
-	const char ** strip_left=NULL;
-	const char ** strip_right=NULL;
-	const char ** prefix=NULL;
-	const char ** suffix=NULL;
+
+	int found_number = 0;
+	int n_r_stripped_save;
+	char * wend_save;
+
+	const char ** strip_left = NULL;
+	const char ** strip_right = NULL;
+	const char ** strip_units = NULL;
+	const char ** prefix = NULL;
+	const char ** suffix = NULL;
 	char word[MAX_WORD+1];
 	char newword[MAX_WORD+1];
-	Dict_node * dn, * dn2, * start_dn;
-	const char * rpunc_con = "RPUNC";
-	const char * lpunc_con = "LPUNC";
-	const char * suf_con = "SUF";
-	const char * pre_con = "PRE";
 
-	if (sent->dict->affix_table!=NULL)
+	const char *r_stripped[MAX_STRIP];  /* these were stripped from the right */
+
+	/* Load affixes from the affix table.  */
+	if (sent->dict->affix_table != NULL)
 	{
-		start_dn = list_whole_dictionary(sent->dict->affix_table->root, NULL);
-		for (dn = start_dn; dn != NULL; dn = dn->right)
-		{
-			if (word_has_connector(dn, rpunc_con, 0)) r_strippable++;
-			if (word_has_connector(dn, lpunc_con, 0)) l_strippable++;
-			if (word_has_connector(dn, suf_con, 0)) s_strippable++;
-			if (word_has_connector(dn, pre_con, 0)) p_strippable++;
-	  	}
-		strip_right = (const char **) xalloc(r_strippable * sizeof(char *));
-		strip_left = (const char **) xalloc(l_strippable * sizeof(char *));
-		suffix = (const char **) xalloc(s_strippable * sizeof(char *));
-		prefix = (const char **) xalloc(p_strippable * sizeof(char *));
+		Dictionary dict = sent->dict->affix_table;
+		r_strippable = dict->r_strippable;
+		l_strippable = dict->l_strippable;
+		u_strippable = dict->u_strippable;
+		p_strippable = dict->p_strippable;
+		s_strippable = dict->s_strippable;
 
-		i=0;
-		j=0;
-		k=0;
-		l=0;
-		dn = start_dn;
-		while (dn != NULL)
-		{
-			if(word_has_connector(dn, rpunc_con, 0)) {
-				strip_right[i] = dn->string;
-				i++;
-			}
-			if(word_has_connector(dn, lpunc_con, 0)) {
-				strip_left[j] = dn->string;
-				j++;
-			}
-			if(word_has_connector(dn, suf_con, 0)) {
-				suffix[k] = dn->string;
-				k++;
-			}
-			if(word_has_connector(dn, pre_con, 0)) {
-				prefix[l] = dn->string;
-				l++;
-			}
-			dn2 = dn->right;
-			dn->right = NULL;
-			xfree(dn, sizeof(Dict_node));
-			dn = dn2;
-		}
+		strip_left = dict->strip_left;
+		strip_right = dict->strip_right;
+		strip_units = dict->strip_units;
+		prefix = dict->prefix;
+		suffix = dict->suffix;
 	}
 
-	for (;;) {
-		for (i=0; i<l_strippable; i++) {
-			if (strncmp(w, strip_left[i], strlen(strip_left[i])) == 0) {
+	for (;;)
+	{
+		for (i=0; i<l_strippable; i++)
+		{
+			/* This is UTF8-safe, I beleive ... */
+			if (strncmp(w, strip_left[i], strlen(strip_left[i])) == 0)
+			{
 				if (!issue_sentence_word(sent, strip_left[i])) return FALSE;
 				w += strlen(strip_left[i]);
 				break;
 			}
 		}
-		if (i==l_strippable) break;
+		if (i == l_strippable) break;
 	}
 
 	/* Now w points to the string starting just to the right of
@@ -347,8 +329,9 @@ static int separate_word(Sentence sent, char *w, char *wend, int is_first_word,
 	 */
 	for (n_r_stripped = 0; n_r_stripped < MAX_STRIP; n_r_stripped++) 
 	{
-		strncpy(word, w, MIN(wend-w, MAX_WORD));
-		word[MIN(wend-w, MAX_WORD)] = '\0';
+		size_t sz = MIN(wend-w, MAX_WORD);
+		strncpy(word, w, sz);
+		word[sz] = '\0';
 		if (wend == w) break;  /* it will work without this */
 
 		if (boolean_dictionary_lookup(sent->dict, word) || is_initials_word(word)) break;
@@ -362,8 +345,9 @@ static int separate_word(Sentence sent, char *w, char *wend, int is_first_word,
 
 			/* the remaining w is too short for a possible match */
 			if ((wend-w) < len) continue;
-			if (strncmp(wend-len, strip_right[i], len) == 0) {
-				r_stripped[n_r_stripped] = i;
+			if (strncmp(wend-len, strip_right[i], len) == 0)
+			{
+				r_stripped[n_r_stripped] = strip_right[i];
 				wend -= len;
 				break;
 			}
@@ -371,13 +355,58 @@ static int separate_word(Sentence sent, char *w, char *wend, int is_first_word,
 		if (i == r_strippable) break;
 	}
 
+	/* Same as above, but with a twist: the only thing that can
+	 * preceed a units suffix is a number. This is so that we can
+	 * split up things like "12ft" (twelve feet) but not split up
+	 * things like "Delft blue". Multiple passes allow for
+	 * constructions such as 12sq.ft.
+	 */
+	n_r_stripped_save = n_r_stripped;
+	wend_save = wend;
+	for (; n_r_stripped < MAX_STRIP; n_r_stripped++) 
+	{
+		size_t sz = MIN(wend-w, MAX_WORD);
+		strncpy(word, w, sz);
+		word[sz] = '\0';
+		if (wend == w) break;  /* it will work without this */
+
+		/* Number */
+		if (is_number(word))
+		{
+			found_number = 1;
+			break;
+		}
+
+		for (i=0; i < u_strippable; i++)
+		{
+			len = strlen(strip_units[i]);
+
+			/* the remaining w is too short for a possible match */
+			if ((wend-w) < len) continue;
+			if (strncmp(wend-len, strip_units[i], len) == 0)
+			{
+				r_stripped[n_r_stripped] = strip_units[i];
+				wend -= len;
+				break;
+			}
+		}
+		if (i == u_strippable) break;
+	}
+
+	/* The root *must* be a number! */
+	if (0 == found_number)
+	{
+		wend = wend_save;
+		n_r_stripped = n_r_stripped_save;
+	}
+
 	/* Now we strip off suffixes...w points to the remaining word, 
 	 * "wend" to the end of the word. */
 
 	s_stripped = -1;
 	strncpy(word, w, MIN(wend-w, MAX_WORD));
 	word[MIN(wend-w, MAX_WORD)] = '\0';
-	word_is_in_dict=0;
+	word_is_in_dict = 0;
 
 	if (boolean_dictionary_lookup(sent->dict, word))
 		word_is_in_dict = 1;
@@ -386,107 +415,99 @@ static int separate_word(Sentence sent, char *w, char *wend, int is_first_word,
 	else if (is_first_word && downcase_is_in_dict (sent->dict,word))
 		word_is_in_dict = 1;
 
-	if(word_is_in_dict==0)
+	if (0 == word_is_in_dict)
 	{
-	  j=0;
-	  for (i=0; i < s_strippable+1; i++) {
-		s_ok = 0;
-		/* Go through once for each suffix; then go through one 
-		 * final time for the no-suffix case */
-		if(i < s_strippable) {
-		  len = strlen(suffix[i]);
-
-		  /* the remaining w is too short for a possible match */
-		  if ((wend-w) < len) continue;
-
-		  if (strncmp(wend-len, suffix[i], len) == 0) s_ok=1;
-				  }
-		else len=0;
-
-		if(s_ok==1 || i==s_strippable)
+		j=0;
+		for (i=0; i <= s_strippable; i++)
 		{
-			strncpy(newword, w, MIN((wend-len)-w, MAX_WORD));
-			newword[MIN((wend-len)-w, MAX_WORD)] = '\0';
+			s_ok = 0;
+			/* Go through once for each suffix; then go through one 
+			 * final time for the no-suffix case */
+			if (i < s_strippable)
+			{
+				len = strlen(suffix[i]);
 
-			/* Check if the remainder is in the dictionary;
-			 * for the no-suffix case, it won't be */
-			if (boolean_dictionary_lookup(sent->dict, newword)) {
-				if(verbosity>1) if(i< s_strippable) printf("Splitting word into two: %s-%s\n", newword, suffix[i]);
-				s_stripped = i;
-				wend -= len;
-				strncpy(word, w, MIN(wend-w, MAX_WORD));
-				word[MIN(wend-w, MAX_WORD)] = '\0';
-				break;
+				/* the remaining w is too short for a possible match */
+				if ((wend-w) < len) continue;
+				if (strncmp(wend-len, suffix[i], len) == 0) s_ok=1;
 			}
+			else
+				len = 0;
 
-			/* If the remainder isn't in the dictionary, 
-			 * try stripping off prefixes */
-		  else {
-			for (j=0; j<p_strippable; j++) {
-			  if (strncmp(w, prefix[j], strlen(prefix[j])) == 0) {
-				strncpy(newword, w+strlen(prefix[j]), MIN((wend-len)-(w+strlen(prefix[j])), MAX_WORD));
-				newword[MIN((wend-len)-(w+strlen(prefix[j])), MAX_WORD)]='\0';
-				if(boolean_dictionary_lookup(sent->dict, newword)) {
-				  if(verbosity>1) if(i < s_strippable) printf("Splitting word into three: %s-%s-%s\n", prefix[j], newword, suffix[i]);
-				  if (!issue_sentence_word(sent, prefix[j])) return FALSE;
-				  if(i < s_strippable) s_stripped = i;
-				  wend -= len;
-				  w += strlen(prefix[j]);
-				  strncpy(word, w, MIN(wend-w, MAX_WORD));
-				  word[MIN(wend-w, MAX_WORD)] = '\0';
-				  break;
+			if (s_ok || i == s_strippable)
+			{
+				strncpy(newword, w, MIN((wend-len)-w, MAX_WORD));
+				newword[MIN((wend-len)-w, MAX_WORD)] = '\0';
+
+				/* Check if the remainder is in the dictionary;
+				 * for the no-suffix case, it won't be */
+				if (boolean_dictionary_lookup(sent->dict, newword))
+				{
+					if ((verbosity>1) && (i < s_strippable))
+						printf("Splitting word into two: %s-%s\n", newword, suffix[i]);
+					s_stripped = i;
+					wend -= len;
+					strncpy(word, w, MIN(wend-w, MAX_WORD));
+					word[MIN(wend-w, MAX_WORD)] = '\0';
+					break;
 				}
-			  }
-			}
-		  }
-		  if(j!=p_strippable) break;
-		}
-	  }
-	}
 
+				/* If the remainder isn't in the dictionary, 
+				 * try stripping off prefixes */
+				else
+				{
+					for (j=0; j<p_strippable; j++)
+					{
+						if (strncmp(w, prefix[j], strlen(prefix[j])) == 0)
+						{
+							int sz = MIN((wend-len)-(w+strlen(prefix[j])), MAX_WORD);
+							strncpy(newword, w+strlen(prefix[j]), sz);
+							newword[sz] = '\0';
+							if (boolean_dictionary_lookup(sent->dict, newword))
+							{
+								if ((verbosity>1) && (i < s_strippable))
+									printf("Splitting word into three: %s-%s-%s\n",
+										prefix[j], newword, suffix[i]);
+								if (!issue_sentence_word(sent, prefix[j])) return FALSE;
+								if (i < s_strippable) s_stripped = i;
+								wend -= len;
+								w += strlen(prefix[j]);
+								sz = MIN(wend-w, MAX_WORD);
+								strncpy(word, w, sz);
+								word[sz] = '\0';
+								break;
+							}
+						}
+					}
+				}
+				if (j != p_strippable) break;
+			}
+		}
+	}
+	
 	/* word is now what remains after all the stripping has been done */
 
-	/*
-	if (n_stripped == MAX_STRIP) {
+	if (n_r_stripped == MAX_STRIP)
+	{
 		prt_error("Error separating sentence.\n"
 		          "\"%s\" is followed by too many punctuation marks.\n", word);
 		return FALSE;
-	} */
+	}
 
 	if (quote_found == TRUE) sent->post_quote[sent->length]=1;
 
 	if (!issue_sentence_word(sent, word)) return FALSE;
 
-	if(s_stripped != -1) {
+	if(s_stripped != -1)
+	{
 	  if (!issue_sentence_word(sent, suffix[s_stripped])) return FALSE;
 	}
 
-	for (i=n_r_stripped-1; i>=0; i--) {
-
-		/* Revert fix r22566, which had a commit message:
-		 * "Fix Bug 9756, crash when grammar checking Word document."
-		 * This fix added the line:
-		 *    if (r_stripped[i] > strlen(*strip_right)) continue;
-		 * However, the addition of this line will break
-		 * the parsing of "Doogie's mother bit her."
-		 *
-		 * The fix is incorrect, because a NULL has been inserted into strip_right,
-		 * making it very short (length 2). Meanwhile, the offset to the 's 
-		 * is 9 chars (greater than 2!)  The string at strip_right[r_stripped[i]]
-		 * is pointing at the 's.
-		 *
-		 * Thus, I'm reverting this fix for now; whatever the problem is,
-		 * it needs to be handled in some other way.
-		 */
-		if (!issue_sentence_word(sent, strip_right[r_stripped[i]])) return FALSE;
+	for (i = n_r_stripped-1; i>=0; i--)
+	{
+		if (!issue_sentence_word(sent, r_stripped[i])) return FALSE;
 	}
 
-	if(sent->dict->affix_table!=NULL) {
-	  xfree(strip_right, r_strippable * sizeof(char *));
-	  xfree(strip_left, l_strippable * sizeof(char *));
-	  xfree(suffix, s_strippable * sizeof(char *));
-	  xfree(prefix, p_strippable * sizeof(char *));
-	}
 	return TRUE;
 }
 
@@ -496,9 +517,9 @@ static int separate_word(Sentence sent, char *w, char *wend, int is_first_word,
  * the sent->word[] array.  Returns TRUE if all is well, FALSE otherwise.
  * Quote marks are treated just like blanks.
  */
-int separate_sentence(char * s, Sentence sent)
+int separate_sentence(const char * s, Sentence sent)
 {
-	char *t;
+	const char *t;
 	int i, is_first, quote_found;
 	Dictionary dict = sent->dict;
 
@@ -510,14 +531,14 @@ int separate_sentence(char * s, Sentence sent)
 
 #ifndef __CYGWIN__
 	/* Reset the multibyte shift state to the initial state */
-	mbtowc(NULL, NULL, 4);
+	i = mbtowc(NULL, NULL, MB_CUR_MAX);
 #endif
 
 	is_first = TRUE;
 	for(;;) 
 	{
 		wchar_t c;
-		int nb = mbtowc(&c, s, 4);
+		int nb = mbtowc(&c, s, MB_CUR_MAX);
 		quote_found = FALSE;
 
 		if (0 > nb) goto failure;
@@ -527,19 +548,19 @@ int separate_sentence(char * s, Sentence sent)
 		{
 			s += nb;
 			if (*s == '\"') quote_found = TRUE;
-			nb = mbtowc(&c, s, 4);
+			nb = mbtowc(&c, s, MB_CUR_MAX);
 			if (0 > nb) goto failure;
 		}
 
 		if (*s == '\0') break;
 
 		t = s;
-		nb = mbtowc(&c, t, 6);
+		nb = mbtowc(&c, t, MB_CUR_MAX);
 		if (0 > nb) goto failure;
 		while (!iswspace(c) && (c != '\"') && (c != 0) && (nb != 0))
 		{
 			t += nb;
-			nb = mbtowc(&c, t, 4);
+			nb = mbtowc(&c, t, MB_CUR_MAX);
 			if (0 > nb) goto failure;
 		}
 
@@ -579,51 +600,53 @@ static int special_string(Sentence sent, int i, const char * s)
 	}
 }
 
-static int guessed_string(Sentence sent, int i, const char * s, const char * type) {
+static int guessed_string(Sentence sent, int i, const char * s, const char * type)
+{
 	X_node * e;
-	char *t, *u;
+	char *t;
 	char str[MAX_WORD+1];
-	if (boolean_dictionary_lookup(sent->dict, type)) {
-	  sent->word[i].x = build_word_expressions(sent, type);
-	  e = sent->word[i].x;
-		  if(is_s_word(s)) {
-
-			for (; e != NULL; e = e->next) {
-			  t = strchr(e->string, '.');
-			  if (t != NULL) {
-				sprintf(str, "%.50s[!].%.5s", s, t+1);
-			  } else {
-				sprintf(str, "%.50s[!]", s);
-			  }
-			  t = (char *) xalloc(strlen(str)+1);
-			  strcpy(t,str);
-			  u = string_set_add(t, sent->string_set);
-			  xfree(t, strlen(str)+1);
-			  e->string = u;
+	if (boolean_dictionary_lookup(sent->dict, type))
+	{
+		sent->word[i].x = build_word_expressions(sent, type);
+		e = sent->word[i].x;
+		if(is_s_word(s))
+		{
+			for (; e != NULL; e = e->next)
+			{
+				t = strchr(e->string, '.');
+				if (t != NULL)
+				{
+					sprintf(str, "%.50s[!].%.5s", s, t+1);
+			  	}
+				else
+				{
+					sprintf(str, "%.50s[!]", s);
+				}
+				e->string = string_set_add(str, sent->string_set);
 			}
-		  }
-
-		  else {
-			if(is_ed_word(s)) {
-			  sprintf(str, "%.50s[!].v", s);
+		}
+		else
+		{
+			if(is_ed_word(s))
+			{
+				sprintf(str, "%.50s[!].v", s);
 			}
-			else if(is_ing_word(s)) {
-			  sprintf(str, "%.50s[!].g", s);
+			else if(is_ing_word(s))
+			{
+				sprintf(str, "%.50s[!].g", s);
 			}
-			else if(is_ly_word(s)) {
-			  sprintf(str, "%.50s[!].e", s);
+			else if(is_ly_word(s))
+			{
+				sprintf(str, "%.50s[!].e", s);
 			}
 			else sprintf(str, "%.50s[!]", s);
 
-			t = (char *) xalloc(strlen(str)+1);
-			strcpy(t,str);
-			u = string_set_add(t, sent->string_set);
-			xfree(t, strlen(str)+1);
-			e->string = u;
-		  }
-		  return TRUE;
-
-	} else {
+			e->string = string_set_add(str, sent->string_set);
+		}
+		return TRUE;
+	}
+	else
+	{
 		err_ctxt ec;
 		ec.sent = sent;
 		err_msg(&ec, Error, 
@@ -634,13 +657,16 @@ static int guessed_string(Sentence sent, int i, const char * s, const char * typ
 	}
 }
 
-static void handle_unknown_word(Sentence sent, int i, char * s) {
-  /* puts into word[i].x the expression for the unknown word */
-  /* the parameter s is the word that was not in the dictionary */
-  /* it massages the names to have the corresponding subscripts */
-  /* to those of the unknown words */
-  /* so "grok" becomes "grok[?].v"  */
-	char *t,*u;
+/**
+ * Puts into word[i].x the expression for the unknown word 
+ * the parameter s is the word that was not in the dictionary 
+ * it massages the names to have the corresponding subscripts 
+ * to those of the unknown words
+ * so "grok" becomes "grok[?].v" 
+ */
+static void handle_unknown_word(Sentence sent, int i, char * s)
+{
+	char *t;
 	X_node *d;
 	char str[MAX_WORD+1];
 
@@ -648,18 +674,18 @@ static void handle_unknown_word(Sentence sent, int i, char * s) {
 	if (sent->word[i].x == NULL)
 		assert(FALSE, "UNKNOWN_WORD should have been there");
 
-	for (d = sent->word[i].x; d != NULL; d = d->next) {
+	for (d = sent->word[i].x; d != NULL; d = d->next)
+	{
 		t = strchr(d->string, '.');
-		if (t != NULL) {
+		if (t != NULL)
+		{
 			sprintf(str, "%.50s[?].%.5s", s, t+1);
-		} else {
+		}
+		else
+		{
 			sprintf(str, "%.50s[?]", s);
 		}
-		t = (char *) xalloc(strlen(str)+1);
-		strcpy(t,str);
-		u = string_set_add(t, sent->string_set);
-		xfree(t, strlen(str)+1);
-		d->string = u;
+		d->string = string_set_add(str, sent->string_set);
 	}
 }
 
@@ -683,7 +709,8 @@ static void handle_unknown_word(Sentence sent, int i, char * s) {
 int build_sentence_expressions(Sentence sent)
 {
 	int i, first_word;  /* the index of the first word after the wall */
-	char *s, *u, temp_word[MAX_WORD+1];
+	char *s, temp_word[MAX_WORD+1];
+	const char * u;
 	X_node * e;
 	Dictionary dict = sent->dict;
 
